package guru.kake.ronove

import guru.kake.bst.BST
import javafx.scene.control.TextField
import javafx.scene.text.FontSmoothingType
import kotlinx.html.*
import kotlinx.html.stream.createHTML
import netscape.javascript.JSObject
import tornadofx.*

/**
 * Tab class to display text using HTML formatting
 */
class ReaderTab : View("Reader") {
    private val cbobject: WebViewCallback = WebViewCallback()
    /** TornadoFX root */
    override val root = vbox {
        webview {
            style {
                fontSmoothingType = FontSmoothingType.GRAY
            }
            engine.loadContent(createHTML().html {
                head {
                    style {
                        unsafe { raw("body { font-size: 48px; }") }
                    }
                }
                body {
                    onLoad = "alert()"
                    p {
                        ruby { +"私"; rt { +"わたし" }}
                        +"の"
                        ruby { +"猫"; rt { +"ねこ" }}
                        +"わかわいいです"
                        ruby("a") {+"b"}
                        a("#") {
                            onClick = "cb.test('foo');"
                            +"linkki"
                        }
                    }

                }
            })

            engine.setOnAlert {
                (engine.executeScript("window") as JSObject).setMember("cb", cbobject)
            }
        }
    }
}

/**
 * Class for receiving events from JS in [ReaderTab]
 */
@Suppress("unused") //IDEA doesn't detect uses of these functions since they are called from JS in the WebView
class WebViewCallback {
    /***/
    fun test(message: String) {
        println(message)
    }
}

/**
 * Tab class for displaying dictionaries
 */
class DictionaryTab : View("Kanji Dictionary") {
    private val ret = DictionaryParser.parseKanjidic2("C:\\repos\\kanjisho\\kanjidic2.xml")
    private val tree = BST(ret)

    /** TornadoFX root */
    override val root = vbox {

        (app as Ronove).cellSelection.addListener(fun(_, _, newSelection) {
            if (newSelection != null) {
                getCharacters(newSelection.original)
            }
        })
        this += searchBox()
    }

    private fun searchBox() = hbox {
        val search = textfield()
        button("Search") {
            action {
                getCharacters(search.text)
            }
        }
    }

    private fun getCharacters(chars: String) {
        root.clear()
        root += searchBox()
        chars.forEach { getCharacter(it.toString()) }
    }

    private fun getCharacter(char: String) {
        val c = tree.find(Character(char))
        if (c != null) {
            val kun: String = c.rmgroups[0].reading.find { it.first == "ja_kun" }?.second ?: ""
            val on: String = c.rmgroups[0].reading.find { it.first == "ja_on" }?.second ?: ""
            val meaning: String = c.rmgroups[0].meaning.find { it.first == null }?.second ?: ""

            root += DictionaryEntry(c.literal, kun, on, meaning)
        }
    }
}

/**
 * TornadoFX fragment to display information on a Kanji
 * @param kanji the kanji
 * @param kun kunyomi reading
 * @param on onyomi reading
 * @param meaning English meaning of the kanji
 */
class DictionaryEntry(
    private val kanji: String,
    private val kun: String,
    private val on: String,
    private val meaning: String
) : Fragment("") {
    /** TornadoFX root */
    override val root = hbox {
        label(kanji) {
            addClass(RonoveStylesheet.bigLabel)
        }
        vbox {
            label("Kun: $kun")
            label("On: $on")
            label(meaning)
        }
    }
}

/**
 * Tab class for displaying original material in image form
 */
class ImageTab : View("Image") {
    /** TornadoFX root */
    override val root = vbox {
        imageview("/yopage.jpg") {
            fitWidth = 300.0
            isPreserveRatio = true
            isSmooth = true
        }
    }
}

/**
 * Tab class for a separate window letting the user run arbitrary strings through machine translation
 */
class TranslatorTab : View("Machine translation") {
    private var fromField: TextField by singleAssign()
    private var toField: TextField by singleAssign()

    //TODO allow switching languages on the fly
    /** TornadoFX root */
    override val root = vbox {
        vbox {
            label("Source language")
            fromField = textfield()
            button("Translate") {
                action {
                    //TranslationBackend.destinationLanguage = "en-US"
                    //TranslationBackend.sourceLanguage = "ja"
                    toField.text = TranslationBackend(fromField.text)
                }
            }
        }
        vbox {
            label("Destination language")
            toField = textfield()
            button("Translate") {
                action {
                    //TranslationBackend.destinationLanguage = "ja"
                    //TranslationBackend.sourceLanguage = "en"
                    fromField.text = TranslationBackend(toField.text)
                }
            }
        }
    }

}